//
//  JDSpeechSynthesizer.h
//  JDSpeechSynthesizer
//
//  Created by 满达 on 2019/12/24.
//  Copyright © 2018年 Beijing Jingdong Shangke Information Technology Co., Ltd. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "JDSpeechSynthesizerDelegate.h"

//! Project version string for JDSpeechSynthesizer.
extern NSString * const JDSpeechSynthesizerVersion;

// In this header, you should import all the public headers of your framework using statements like #import <JDSpeechSynthesizer/PublicHeader.h>

NS_ASSUME_NONNULL_BEGIN

@interface JDSpeechSynthesizer : NSObject

/*!
 *  设置识别的委托对象
 */
@property (nonatomic, weak) id<JDSpeechSynthesizerDelegate> delegate;

/*!
 *  返回合成对象的单例
 *
 *  @return 合成对象
 */
+ (instancetype) sharedInstance;

/*!
 *  销毁合成对象。
 *
 *  @return 成功返回YES,失败返回NO.
 */
+ (BOOL) destroy;

/*!
 *  设置合成参数
 *
 *  @param value 参数取值
 *  @param key   合成参数
 *  详细参考JDSTTSParameters.h
 *
 *  @return 设置成功返回YES，失败返回NO
 */
-(BOOL) setParameter:(id) value forKey:(NSString*)key;

/*!
 *  获取合成参数
 *
 *  @param key 参数key
 *  详细参考JDSTTSParameters.h
 *
 *  @return 参数值
 */
-(id) parameterForKey:(NSString *)key;

/*!
 *  开始合成(播放)<br>
 *  调用此函数进行合成，如果发生错误会回调错误`onCompleted`
 *
 *  @param text 合成的文本,最大的字节数为1k
 */
- (void) startSpeaking:(NSString *)text;

/*!
 *  开始合成(不播放)<br>
 *  调用此函数进行合成，如果发生错误会回调错误`onCompleted`
 *
 *  @param text 合成的文本,最大的字节数为1k
 */
- (void) synthesize:(NSString *)text;

/*!
 *  暂停播放<br>
 *  暂停播放之后，合成不会暂停，仍会继续，如果发生错误则会回调错误`onCompleted`
 */
- (void) pauseSpeaking;

/*!
 *  恢复播放
 */
- (void) resumeSpeaking;

/*!
 *  停止播放并停止合成
 */
- (void) stop;

/*!
 *  是否正在播放
 */
@property (nonatomic, readonly) BOOL isSpeaking;

@end

NS_ASSUME_NONNULL_END
