//
//  JDSpeechSynthesizerDelegate.h
//  JDSpeechSynthesizer
//
//  Created by 满达 on 2019/12/24.
//  Copyright © 2018年 Beijing Jingdong Shangke Information Technology Co., Ltd. All rights reserved.
//

#ifndef JDSpeechSynthesizerDelegate_h
#define JDSpeechSynthesizerDelegate_h

#import <Foundation/Foundation.h>

@class JDSpeechError;

/*!
 *  语音合成回调
 */
@protocol JDSpeechSynthesizerDelegate <NSObject>

@required

/*!
 *  结束回调<br>
 *  当整个合成结束之后会回调此函数
 *
 *  @param error 错误码
 */
- (void) onCompleted:(JDSpeechError*) error;


@optional

/*!
*   开始合成回调
*/
- (void) onBegin;

/*!
*  合成数据回调
*/

- (void) onSynthesizeData:(NSData*) data timeStamp:(NSString *) timeStamp;

/*!
 *  合成进度回调
 *
 *  @param progress 缓冲进度，0-100
 *  @param msg      附件信息，此版本为nil
*/

- (void) onSynthesizeProgress:(float) progress message:(NSString *)msg;

/*!
 *  开始播放回调
 */
- (void) onSpeakBegin;

/*!
 *  播放进度回调
 *
 *  @param progress 当前播放进度，0-100
 *  @param msg      附件信息，此版本为nil
 */
- (void) onSpeakProgress:(float) progress message:(NSString *)msg;

/*!
 *  结束播放回调
 */
- (void) onSpeakEnd;

/*!
 *  缓冲开始回调
 *
 *  @param msg      附件信息，此版本为nil
 */
- (void) onBufferStart:(NSString *)msg;

/*!
 *  缓冲技术回调
 *
 *  @param msg      附件信息，此版本为nil
 */
- (void) onBufferEnd:(NSString *)msg;

/*!
 *  创建完语音合成实例回调，只有在离线引擎加载前端资源文件以及模型文件的时候回调
 *
 *  @param msg      附件信息，此版本为nil
 */
- (void) onSynthesizerCreated:(NSString *)msg;

/*!
 *  设置参数回到，只有在离线引擎加载前端资源文件以及模型文件的时候回调
 *
 *  @param key     设置参数的关键字
 *  @param error 错误码
 */
- (void) onParamSetted:(NSString*)key err:(JDSpeechError*) error;

@end

#endif /* JDSpeechSynthesizerDelegate_h */
