//
//  JDViewController.m
//  JD_Speech_Test
//
//  Created by 满达 on 2018/6/9.
//  Copyright © 2018年 满达. All rights reserved.
//

#import "JDViewController.h"
#import <JDSpeechSynthesizer/JDSpeechSynthesizer.h>
#import <JDSpeechSynthesizer/JDSTTSDefines.h>
#import <JDSpeechSynthesizer/JDSTTSParameters.h>
#import <JDSpeechSynthesizer/JDSpeechError.h>
#import "TTSConfig.h"
#import "CommonUtil.h"
#import <iconv.h>

#pragma mark - defines

// Default values
static NSString *const kTextDef                = @"你好,京东！";
static const NSInteger kSampleRateDef          = EJDS_TTSSampleRate24K;
static const float     kVolumeLevelDef         = 2.0f;
static const float     kSpeedLevelDef          = 1.0f;
static NSString *const kApplicationIdDef       = @"";
static NSString *const kSecretKeyDef           = @"";
static NSString *const kApplicationKeyDef      = @"";
static NSString *const kServerURLDef           = @"";
static const NSInteger kLanguageDef            = EJDS_TTSLanguageMandarin;
static const NSInteger kTimDef                 = EJDS_TTS_TAOTAO_FEMALE;
static const NSInteger kAueDef                 = EJDS_TTSAudioFormatOPUS;
static const NSInteger kTimeStampDef           = ETTSTimeStampEna;
static const NSInteger kTaskDef                = 0; // 0：播报 1：合成
static const NSInteger kEngineDef              = 0; // 0：离线 1：在线

static NSString *const JDS_TTS_TASK_TYPE       = @"TTS.TaskType";
static NSString *const kFEModelName            = @"jd_tts_text.dat";
static NSString *const kModelName              = @"binbin.dat";

static NSString *const kIsSaveFileKey          = @"JDTTS_SPEECH_AUTO.SAVEFILE";
static const BOOL      kIsSaveFileDef          = NO;

static NSString *const kAudioInputDef          = @"JDAudioInput";    // 应用程序Documents目录下
static NSString *const kAudioOutputDef         = @"JDAudioOutput";   // 应用程序Documents目录下

#define __TEST
//#undef __TEST

#ifdef __TEST
// Testing
// APP ID调用NeuHub平台无需设置，具体特殊需求发送到aitts-contact@jd.com进行申请
static NSString *const kApplicationIdTestDef   = @"7a12e9ac9a82670a2fe986bf5204d1**";
static NSString *const kApplicationKeyTestDef  = @"894866b6ce511b438a974bdaa98b61**";
static NSString *const kSecretKeyTestDef       = @"a6eba1a3cedf62cf0347d644062471**";
static NSString *const kServerURLTestDef       = @"https://aiapi.jd.com/jdai/tts";
// 精品音色 https://aiapi.jd.com/jdai/tts_vip
// 虚拟主播 https://aiapi.jd.com/jdai/tts_virtual
// 京东云新网关 https://aiapi.jd.com/jdai/tts
// NeuHub接口 http://tts-stream.jd.com/tts
#endif

#define __SAVE_TO_FILE
//#undef __SAVE_TO_FILE

@interface JDViewController () <JDSpeechSynthesizerDelegate> {
    BOOL                  _isRunning;
    BOOL                  _isPlaying;
    
    unsigned long long    _statLocation;
    unsigned long long    _fileLength;
    
    BOOL                  _isSaveToFile;
    NSString*             _preText;
    
    // 离线引擎相关的前端资源文件以及模型文件
    __block BOOL                  _femodelLoaded;
    __block BOOL                  _modelLoaded;
}

// Outlets
@property (weak, nonatomic) IBOutlet UITextView *ttsTextView;
@property (weak, nonatomic) IBOutlet UITextView *logsTextView;

@property (weak, nonatomic) IBOutlet UISegmentedControl *sampleRateChoice;
@property (weak, nonatomic) IBOutlet UITextField *APPID;
@property (weak, nonatomic) IBOutlet UITextField *AppKey;
@property (weak, nonatomic) IBOutlet UITextField *SecretKey;
@property (weak, nonatomic) IBOutlet UITextField *serverURL;
@property (weak, nonatomic) IBOutlet UISegmentedControl *language;
@property (weak, nonatomic) IBOutlet UISegmentedControl *timbre;
@property (weak, nonatomic) IBOutlet UISegmentedControl *aueChoice;
@property (weak, nonatomic) IBOutlet UISlider *volumeSlider;
@property (weak, nonatomic) IBOutlet UISlider *speedSlider;
@property (weak, nonatomic) IBOutlet UILabel *volumeRes;
@property (weak, nonatomic) IBOutlet UILabel *speedRes;
@property (weak, nonatomic) IBOutlet UISwitch *timeStampSwitch;
@property (weak, nonatomic) IBOutlet UISegmentedControl *taskChoice;
@property (weak, nonatomic) IBOutlet UISwitch *isSaveFile;
@property (weak, nonatomic) IBOutlet UISegmentedControl *engineType;

// Actions
- (IBAction)sampleRateDidChoice:(id)sender;
- (IBAction)APPIDEdit:(id)sender;
- (IBAction)APPKeyEdit:(id)sender;
- (IBAction)SecretKeyEdit:(id)sender;
- (IBAction)serverURLEdit:(id)sender;
- (IBAction)languageDidChoice:(id)sender;
- (IBAction)timbreDidChoice:(id)sender;
- (IBAction)aueDidChoice:(id)sender;
- (IBAction)setVolume:(id)sender;
- (IBAction)setSpeed:(id)sender;
- (IBAction)timeStampSwitch:(id)sender;
- (IBAction)taskChoice:(id)sender;
- (IBAction)toggleSaveFile:(id)sender;
- (IBAction)engineChoice:(id)sender;

- (IBAction)defaultSet:(id)sender;

//-----------------------------------------------------------------------------------
@property (nonatomic,strong) JDSpeechSynthesizer  *speechSynthesizer;

@property (weak, nonatomic) IBOutlet UIButton *startBtn;
@property (weak, nonatomic) IBOutlet UIButton *stopBtn;
@property (weak, nonatomic) IBOutlet UIButton *loadModel;

#ifdef __SAVE_TO_FILE
@property(atomic,strong)  NSString      *outfile;
#endif

@property (atomic, assign)   NSInteger           taskCount;    // 目前运行任务数
@property (atomic, assign)   NSInteger           startCount;   // 开始提交任务数
@property (atomic, assign)   NSInteger           synPlayIndex; // 播放还是合成任务类型

@property(atomic, strong)   NSMutableArray       *allFiles;
@property(atomic,strong)    NSString             *processResfile;
@property(atomic)   NSInteger                    currentIndex;
@property(nonatomic, strong) dispatch_queue_t queueTask;
@property(atomic)     NSTimeInterval timestampStart;
@property(atomic)     NSTimeInterval timestampResponse;
@property(atomic)     NSTimeInterval timestampEnd;

- (void)mySetDefault;
- (void)myViewLoad;
@end

@implementation JDViewController

#pragma mark --defaultMethods
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    // 启动程序的时候预先加载的设置值
    [self myViewLoad];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];

    [self initializeBtn];
    _isRunning = NO;
    _isPlaying = NO;
}

- (void) viewWillDisappear:(BOOL)animated {
    //NSLog(@"viewWillDisappear");
    if (_speechSynthesizer) {
        // 如果后面继续使用stop即可
        //[_speechSynthesizer stop];
        // 如果不用合成对象销毁即可
        [JDSpeechSynthesizer destroy];
        _speechSynthesizer = nil;
    }
    _isRunning = NO;
    _isPlaying = NO;
}

#pragma mark - Initialization

- (JDSpeechSynthesizer *) speechSynthesizer {
    //TTS singleton
    if (_speechSynthesizer == nil) {
        _speechSynthesizer = [JDSpeechSynthesizer sharedInstance];
        [self printLogs:[NSString stringWithFormat:@"TTS版本：%@", JDSpeechSynthesizerVersion]];
        _speechSynthesizer.delegate = self;
    }
    
    // 可以参考JDSTTSParameters.h进行定义
    // 没有更改的参数项，无需设置
    [self setConfigs];
    
    return _speechSynthesizer;
}

#pragma mark --Operations

- (void) initializeBtn {
    self.startBtn.enabled = YES;
    self.stopBtn.enabled  = NO;
}

- (void) startBtnTouch {
    self.startBtn.enabled = NO;
    self.stopBtn.enabled  = YES;
}

- (void) stopBtnTouch {
    self.startBtn.enabled = YES;
    self.stopBtn.enabled  = NO;
}

- (IBAction)start:(id)sender {
    [self userConfigs];
    TTSConfig *instance = [TTSConfig sharedInstance];
    if (instance.ttsEngineType == EJDS_TTSLocal) {
        if (!_femodelLoaded || !_modelLoaded) {
            NSLog(@"haven't loaded model");
            [self printLogs:@"haven't loaded model"];
            return;
        }
    }
    if (_isRunning) {
        NSLog(@"Already Running");
        [self printResult:@"Already Running"];
        return;
    }
    _isRunning = YES; // _isRunning针对整个请求任务，而非单个请求
    
    [self startBtnTouch];
    [self cleanLogUI];
    
    // checkout the JDAudioInput and JDAudioOutput directory under the document
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if (![fileManager fileExistsAtPath:[CommonUtil documentsPath:kAudioInputDef]]) {
        NSLog(@"Input dir invalid:%@", kAudioInputDef);
        [self printResult:[NSString stringWithFormat:@"Input dir invalid:%@", kAudioInputDef]];
        [fileManager createDirectoryAtPath:[CommonUtil documentsPath:kAudioInputDef] withIntermediateDirectories:YES attributes:nil error:nil];
        _isRunning = NO;
        return;
    }
    // /Users/manda1/Library/Containers/maccatalyst.com.jd.JDSpeech/Data/Documents
    if (![fileManager fileExistsAtPath:[CommonUtil documentsPath:kAudioOutputDef]]) {
        NSLog(@"Output dir invalid:%@", kAudioOutputDef);
        [self printResult:[NSString stringWithFormat:@"Output dir invalid:%@", kAudioOutputDef]];
        [fileManager createDirectoryAtPath:[CommonUtil documentsPath:kAudioOutputDef] withIntermediateDirectories:YES attributes:nil error:nil];
        _isRunning = NO;
        return;
    }
    
    [self creatProcessFile:kAudioOutputDef];
    
    NSLog(@"---List all text files---");
    [self printResult:@"---List all text files---"];
    // 枚举文件
    [self.allFiles removeAllObjects];
    self.allFiles = nil;
    self.allFiles = [[NSMutableArray alloc] init];
    NSString *fileFormate = @"txt";
    [self listFiles:kAudioInputDef withFormat:fileFormate];
    if ([self.allFiles count] <= 0) {
        NSLog(@"No files in %@", kAudioInputDef);
        [self printResult:[NSString stringWithFormat:@"No files in %@", kAudioInputDef]];
        _isRunning = NO;
        return;
    }
    NSLog(@"-----------------");
    [self printResult:@"-----------------"];
    [self outToProcessFile:[@"-----------------\r\n" dataUsingEncoding:NSUTF8StringEncoding]];
    
    _currentIndex      = 0;
    _statLocation      = 0;
    _fileLength        = 0;
    
    _taskCount         = 0;
    _startCount        = 0;
    if ([self.allFiles count] > 0) {
        _fileLength = [self getFileSize:[self getFilePathByIndex:_currentIndex]];

        self.queueTask = nil;
        self.queueTask = dispatch_queue_create("com.jd.JDSpeech.AutoTest", NULL);
        [self runTask:NO];
    }
}

- (IBAction)stop:(id)sender {
    if (!_isRunning) {
        NSLog(@"Haven't Running");
        [self printResult:@"Haven't Running"];
        return;
    }
    if (_isRunning && _speechSynthesizer) {
        [_speechSynthesizer stop];
    }
    _isRunning = NO;
    _isPlaying = NO;
    [self stopBtnTouch];
}

- (void)runTask:(BOOL)rerun  {
    if (!_isRunning)
        return;
    
    if (self.currentIndex >= [self.allFiles count] && _fileLength == 0) {
        _isRunning = NO;
        NSLog(@"Process Finished");
        [self printResult:@"Process Finished"];
        return;
    }
    
    self.ttsTextView.text  = @"";
    self.logsTextView.text = @"";
    __weak typeof(self) weakSelf = self;
    dispatch_sync(self.queueTask, ^{
        __strong typeof(self) strongSelf = weakSelf;
        
        NSString *text;
        if (!rerun) {
            text = [strongSelf readLineTextFile];
            _preText = text;
        } else {
            text = _preText;
        }
        if ([CommonUtil isBlankString:text]) {
            NSLog(@"No text to process");
            text = @"";
        }
        
        strongSelf.timestampStart = [NSDate timeIntervalSinceReferenceDate];
        NSLog(@"[%f]Start run:%@", strongSelf.timestampStart, text);
        [strongSelf printResult:[NSString stringWithFormat:@"[%f]Start run:%@",strongSelf.timestampStart, text]];
        [strongSelf outToProcessFile:[[NSString stringWithFormat:@"[%f]Start run:%@",strongSelf.timestampStart, text] dataUsingEncoding:NSUTF8StringEncoding]];

        TTSConfig *instance = [TTSConfig sharedInstance];
#ifdef __SAVE_TO_FILE
        // 创建一个file
        if (instance.saveFileVal)
            [strongSelf creatOutFile];
#endif
        NSInteger index = kTaskDef;
        if (instance != nil) {
            index = instance.taskTypeVal;
        }
        _synPlayIndex = index;
        _taskCount++;
        if (_startCount < 3) {
            _startCount++;
            NSLog(@"Ryan start count:%ld", _startCount);
        }
        [text stringByReplacingOccurrencesOfString:@"\n" withString:@""];
        [text stringByReplacingOccurrencesOfString:@"\r" withString:@""];
        if (index == 0) {
            [self.speechSynthesizer startSpeaking:text];
        } else {
            [self.speechSynthesizer synthesize:text];
        }
    });
}
- (IBAction)loadModel:(id)sender {
    [self userConfigs];
    TTSConfig *instance = [TTSConfig sharedInstance];
    if (instance.ttsEngineType != EJDS_TTSLocal) {
        return;
    }
    if (!_speechSynthesizer) {
        // 涉及到前端资源以及离线模型加载，可以另外以其他异步机制加载
        _speechSynthesizer = [JDSpeechSynthesizer sharedInstance];
        //[self printLogs:[NSString stringWithFormat:@"TTS版本：%@", JDSpeechSynthesizerVersion]];
        _speechSynthesizer.delegate = self;
    }
    NSString *feModel = [CommonUtil documentsPath:kFEModelName];
    NSString *tts_res_path = [_speechSynthesizer parameterForKey:JDS_TTS_RES_PATH];
    NSString *model   = [CommonUtil documentsPath:kModelName];
    NSString *tts_voice_path = [_speechSynthesizer parameterForKey:JDS_TTS_VOICE_PATH];

    if (![CommonUtil fileExist:feModel]) {
        NSLog(@"No exist:%@", feModel);
        [self printLogs:[NSString stringWithFormat:@"No exist:%@", feModel]];
    }
    
    if (![CommonUtil fileExist:model]) {
        NSLog(@"No exist:%@", model);
        [self printLogs:[NSString stringWithFormat:@"No exist:%@", model]];
    }

    if ([CommonUtil fileExist:feModel] && ![feModel isEqualToString:tts_res_path]) {
        [_speechSynthesizer setParameter:feModel forKey:JDS_TTS_RES_PATH];
    }
    if ([CommonUtil fileExist:model] && ![model isEqualToString:tts_voice_path]) {
        [_speechSynthesizer setParameter:model forKey:JDS_TTS_VOICE_PATH];
    }
}

#pragma mark - implementation delegate

- (void) onCompleted:(JDSpeechError*) error {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<合成结束回调");
            [self printLogs:@"<<合成结束回调"];
            [self printLogs:[NSString stringWithFormat:@"合成结束结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc]];
            NSLog(@"%@",[NSString stringWithFormat:@"合成结束结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc]);
            [strongSelf outToProcessFile:[[NSString stringWithFormat:@"合成结束结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc] dataUsingEncoding:NSUTF8StringEncoding]];
            
            strongSelf.timestampEnd = [NSDate timeIntervalSinceReferenceDate];
            NSLog(@"Duration time:%f", strongSelf.timestampEnd - strongSelf.timestampStart);
            [strongSelf printLogs:[NSString stringWithFormat:@"Duration time:%f", strongSelf.timestampEnd - strongSelf.timestampStart]];
            [strongSelf printResult:[NSString stringWithFormat:@"Duration time:%f", strongSelf.timestampEnd - strongSelf.timestampStart]];
            [strongSelf outToProcessFile:[[NSString stringWithFormat:@"Duration time:%f\r\n", strongSelf.timestampEnd - strongSelf.timestampStart] dataUsingEncoding:NSUTF8StringEncoding]];
            
            if (strongSelf->_synPlayIndex == 1) {
                strongSelf->_taskCount--;
                NSLog(@"Ryan syn _taskCount:%ld", strongSelf->_taskCount);
            }
            if (error.errorCode != ER_JDS_TTS_CacheBufferLimit &&
                error.errorCode != ER_JDS_TTS_SynRunning &&
                strongSelf->_startCount < 3) {
                [strongSelf runTask:NO];
            } else if (error.errorCode == ER_JDS_TTS_CacheBufferLimit || error.errorCode == ER_JDS_TTS_SynRunning) {
                NSLog(@"未播放缓冲达到上限或当前合成任务未完成");
                [self printLogs:@"未播放缓冲达到上限或当前合成任务未完成"];
                if (strongSelf->_synPlayIndex == 1 && strongSelf->_taskCount < 3) {
                    [strongSelf runTask:YES];
                }
            } else {
                if (strongSelf->_synPlayIndex == 1 && strongSelf->_taskCount < 3) {
                    [strongSelf runTask:NO];
                }
            }
        }
    });
}

- (void) onBegin {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<开始合成回到");
            [self printLogs:@"<<开始合成回调"];
            NSTimeInterval timestamp = [NSDate timeIntervalSinceReferenceDate];
            [strongSelf printLogs:[NSString stringWithFormat:@"开始合成:%f", timestamp - strongSelf.timestampStart]];
            [strongSelf printResult:[NSString stringWithFormat:@"开始合成:%f", timestamp - strongSelf.timestampStart]];
            [strongSelf outToProcessFile:[[NSString stringWithFormat:@"开始合成:%f\r\n", timestamp - strongSelf.timestampStart] dataUsingEncoding:NSUTF8StringEncoding]];
        }
    });
}

- (void) onSynthesizeData:(NSData*) data timeStamp:(NSString *) timeStamp {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<合成语音数据回调");
            [self printLogs:@"<<合成语音数据回调"];
            if (![CommonUtil isBlankString:timeStamp]) {
                NSLog(@"timeStamp:%@", timeStamp);
                [strongSelf printLogs:[NSString stringWithFormat:@"timeStamp:%@", timeStamp]];
            }
            strongSelf.timestampResponse = [NSDate timeIntervalSinceReferenceDate];
            NSLog(@"Response time:%f", strongSelf.timestampResponse - strongSelf.timestampStart);
            [strongSelf printLogs:[NSString stringWithFormat:@"Response time:%f", strongSelf.timestampResponse - strongSelf.timestampStart]];
            [strongSelf outToProcessFile:[[NSString stringWithFormat:@"Response time:%f\r\n", strongSelf.timestampResponse - strongSelf.timestampStart] dataUsingEncoding:NSUTF8StringEncoding]];
            
#ifdef __SAVE_TO_FILE
            // Write to file
            if (data && data.length > 0) {
                NSLog(@"data length: %ld", (unsigned long)data.length);
                [strongSelf outToFile:data];
            }
#endif
        }
    });
}

- (void) onSynthesizeProgress:(float) progress message:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<合成进度回调");
            [self printLogs:@"<<合成进度回调"];
            [self printLogs:[NSString stringWithFormat:@"合成进度：%f",progress]];
            NSLog(@"合成进度：%f",progress);
        }
    });
}

- (void) onSpeakBegin {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            strongSelf->_isPlaying = YES;
            NSTimeInterval timestamp = [NSDate timeIntervalSinceReferenceDate];
            NSLog(@"<<开始播报回调");
            [self printLogs:[NSString stringWithFormat:@"开始播报:%f", timestamp - strongSelf.timestampStart]];
            [strongSelf printResult:[NSString stringWithFormat:@"开始播报:%f", timestamp - strongSelf.timestampStart]];
            [strongSelf outToProcessFile:[[NSString stringWithFormat:@"开始播报:%f\r\n", timestamp - strongSelf.timestampStart] dataUsingEncoding:NSUTF8StringEncoding]];
        }
    });
}

- (void) onSpeakProgress:(float) progress message:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<播报进度回调");
            [self printLogs:@"<<播报进度回调"];
            [self printLogs:[NSString stringWithFormat:@"播报进度：%f",progress]];
            NSLog(@"播报进度：%f",progress);
        }
    });
}

- (void) onSpeakEnd {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            strongSelf->_isPlaying = NO;
            NSLog(@"<<结束播报回调");
            NSTimeInterval timestamp = [NSDate timeIntervalSinceReferenceDate];
            [self printLogs:@"<<结束播报回调"];
            [self printLogs:[NSString stringWithFormat:@"结束播报:%f", timestamp - strongSelf.timestampStart]];
            [strongSelf printResult:[NSString stringWithFormat:@"结束播报:%f", timestamp - strongSelf.timestampStart]];
            [strongSelf outToProcessFile:[[NSString stringWithFormat:@"结束播报:%f\r\n", timestamp - strongSelf.timestampStart] dataUsingEncoding:NSUTF8StringEncoding]];
            
            strongSelf->_taskCount--;
            NSLog(@"Ryan play _taskCount:%ld", strongSelf->_taskCount);
            if (strongSelf->_startCount >= 3 && strongSelf->_taskCount < 3) {
                [strongSelf runTask:NO];
            }
        }
    });
}

- (void) onBufferStart:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<缓冲显示回调");
            [self printLogs:@"<<缓冲显示回调"];
            [strongSelf printResult:@"<<缓冲显示回调"];
            [strongSelf outToProcessFile:[[NSString stringWithFormat:@"<<缓冲显示回调"] dataUsingEncoding:NSUTF8StringEncoding]];
        }
    });
}

- (void) onBufferEnd:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<缓冲隐藏回调");
            [self printLogs:@"<<缓冲隐藏回调"];
            [strongSelf printResult:@"<<缓冲隐藏回调"];
            [strongSelf outToProcessFile:[[NSString stringWithFormat:@"<<缓冲隐藏回调"] dataUsingEncoding:NSUTF8StringEncoding]];
        }
    });
}

- (void) onParamSetted:(NSString*)key err:(JDSpeechError*) error {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            [self printLogs:[NSString stringWithFormat:@"设置参数结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc]];
            NSLog(@"%@",[NSString stringWithFormat:@"设置参数结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc]);
            if (error.errorCode == 0 && [key isEqualToString:JDS_TTS_RES_PATH]) {
                strongSelf->_femodelLoaded = YES;
            }
            if (error.errorCode == 0 && [key isEqualToString:JDS_TTS_VOICE_PATH]) {
                strongSelf->_modelLoaded = YES;
            }
        }
    });
}

- (void)cleanLogUI {
    self.ttsTextView.text     = @"";
    self.logsTextView.text    = @"";
}

#pragma mark - read Text file

- (NSString*) readLineTextFile {
    NSMutableString *textLine = [[NSMutableString alloc] init];
    if (self.currentIndex >= [self.allFiles count]) {
        return textLine;
    }
    
    NSString *textFile = [self getFilePathByIndex:self.currentIndex];
    NSLog(@"textFile:%@", textFile);
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForReadingAtPath:textFile];
    if (nil == fileHandle)
        return textLine;
    
    NSUInteger len = 1024;
    while (_statLocation < _fileLength) {
        [fileHandle seekToFileOffset:_statLocation];
        unsigned long long readLen = len;
        if ((_statLocation + len) >= _fileLength) {
            readLen = _fileLength - _statLocation;
        }
        
        NSData *data = [fileHandle readDataOfLength:(NSUInteger)readLen];
        
        if (data && data.length > 0) {
            //NSStringEncoding myEncoding = CFStringConvertEncodingToNSStringEncoding (kCFStringEncodingGB_18030_2000);
            //NSString *dataText = [[NSString alloc] initWithData:data encoding:myEncoding];
            //NSString *dataText = [[NSString alloc] initWithData:[self cleanUTF8:data] encoding:NSUTF8StringEncoding];
            NSString *dataText = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
            NSLog(@"dataText:%@", dataText);
            if ([CommonUtil isBlankString:dataText]) {
                NSLog(@"dataText is blank");
                [self printResult:@"dataText is blank"];
                _statLocation += readLen;
                continue;
            }
            NSRange range = [dataText rangeOfString:@"。"];
            if (!(range.location == NSNotFound && range.length == 0)) {
                NSString *text = [dataText substringWithRange:NSMakeRange(0, range.location + range.length)];
                [textLine appendString:text];
                _statLocation += [text lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
                break;
            }
            range = [dataText rangeOfString:@"？"];
            if (!(range.location == NSNotFound && range.length == 0)) {
                NSString *text = [dataText substringWithRange:NSMakeRange(0, range.location + range.length)];
                [textLine appendString:text];
                _statLocation += [text lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
                break;
            }
            range = [dataText rangeOfString:@"！"];
            if (!(range.location == NSNotFound && range.length == 0)) {
                NSString *text = [dataText substringWithRange:NSMakeRange(0, range.location + range.length)];
                [textLine appendString:text];
                _statLocation += [text lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
                break;
            }
            range = [dataText rangeOfString:@"\n"];
            if (range.location == NSNotFound && range.length == 0) {
                [textLine appendString:dataText];
                _statLocation += [dataText lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
            } else {
                NSString *text = [dataText substringWithRange:NSMakeRange(0, range.location + range.length)];
                [textLine appendString:text];
                _statLocation += [text lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
                break;
            }
        }
    }
    
    [fileHandle closeFile];
    
    if (_statLocation >= _fileLength) {
        self.currentIndex++;
        _statLocation      = 0;
        _fileLength        = 0;
        if (self.currentIndex < [self.allFiles count]) {
            _fileLength        = [self getFileSize:[self getFilePathByIndex:self.currentIndex]];
        }
    }
    
    return textLine;
}

/*- (NSData *)cleanUTF8:(NSData *)data {
    iconv_t cd = iconv_open("UTF-8", "UTF-8"); // 从utf8转utf8
    int one = 1;
    iconvctl(cd, ICONV_SET_DISCARD_ILSEQ, &one); // 丢弃不正确的字符
    
    size_t inbytesleft, outbytesleft;
    inbytesleft = outbytesleft = data.length;
    char *inbuf  = (char *)data.bytes;
    char *outbuf = (char *)malloc(sizeof(char) * data.length);
    char *outptr = outbuf;
    if (iconv(cd, &inbuf, &inbytesleft, &outptr, &outbytesleft)
        == (size_t)-1) {
        NSLog(@"this should not happen, seriously");
        return nil;
    }
    NSData *result = [NSData dataWithBytes:outbuf length:data.length - outbytesleft];
    iconv_close(cd);
    free(outbuf);
    return result;
}*/

#pragma mark - private operations

- (void)listFiles:(NSString *)audioSourcePath withFormat:(NSString *)format {
    //文件操作对象
    NSFileManager *manager = [NSFileManager defaultManager];
    
    //所查找文件夹的路径
    NSString * path = [CommonUtil documentsPath:audioSourcePath];
    NSLog(@"Input Directory:%@", path);
    [self outToProcessFile:[[NSString stringWithFormat:@"Input Directory:%@\n", path] dataUsingEncoding:NSUTF8StringEncoding]];
    
    NSString *audioSource = [path stringByExpandingTildeInPath];
    
    //目录迭代器
    NSDirectoryEnumerator *direnum = [manager enumeratorAtPath:audioSource];
    
    //快速枚举
    for (NSString *filename in direnum) {
        if ([[filename pathExtension] isEqualToString:format]) {
            [self.allFiles addObject:filename];
        }
    }
    
    NSLog(@"%lu %@ files:",(unsigned long)[self.allFiles count],format);
    [self printResult:[NSString stringWithFormat:@"%lu %@ files:", (unsigned long)[self.allFiles count], format]];
    [self outToProcessFile:[[NSString stringWithFormat:@"%lu %@ files:\n", (unsigned long)[self.allFiles count], format] dataUsingEncoding:NSUTF8StringEncoding]];
    
    for (NSString *filename in self.allFiles) {
        unsigned long long fileSize = [self getFileSize:[self getFilePathByName:filename]];
        [self printResult:[NSString stringWithFormat:@"%@ size:%@",filename, @(fileSize)]];
        NSLog(@"%@",[NSString stringWithFormat:@"%@ size:%@",filename, @(fileSize)]);
        [self outToProcessFile:[[NSString stringWithFormat:@"%@ size:%@\n",filename, @(fileSize)] dataUsingEncoding:NSUTF8StringEncoding]];
    }
}

- (NSString *)getFilePathByName:(NSString *)name {
    NSString *textFile = [NSString stringWithFormat:@"%@/%@",[CommonUtil documentsPath:kAudioInputDef], name];
    return textFile;
}

- (NSString *)getFilePathByIndex:(NSInteger)index {
    if (index >= [self.allFiles count]) {
        return nil;
    }
    NSString *textFile = [NSString stringWithFormat:@"%@/%@",[CommonUtil documentsPath:kAudioInputDef], [self.allFiles objectAtIndex:index]];
    
    return textFile;
}

- (unsigned long long) getFileSize:(NSString *)path {
    NSFileManager *fileManager = [[NSFileManager alloc] init];
    
    if ([fileManager fileExistsAtPath:path]) {
        NSDictionary *fileDic = [fileManager attributesOfItemAtPath:path error:nil];
        unsigned long long size = [[fileDic objectForKey:NSFileSize] longLongValue];
        return size;
    }
    return 0;
}

#pragma mark - log to file of Result

- (void) creatProcessFile:(NSString *)audioOutput {
    
    //创建文件管理器
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    //获取路径
    //参数 要获取的哪种路径
    NSString *paths = [CommonUtil documentsPath:audioOutput];
    
    //更改到操作的目录下
    [fileManager changeCurrentDirectoryPath:[paths stringByExpandingTildeInPath]];
    
    NSTimeInterval timestamp = [NSDate timeIntervalSinceReferenceDate];
    //创建文件fileName文件名称，contents文件内容，如果开始没有内容可以设置为nil，attributes文件的属性,初始为nil
    [fileManager createFileAtPath:[NSString stringWithFormat:@"%f.txt",timestamp] contents:nil attributes:nil];
    
    //获取文件路径
    self.processResfile = [paths stringByAppendingPathComponent:[NSString stringWithFormat:@"%f.txt",timestamp]];
    NSLog(@"Process result file:%@", self.processResfile);
}

- (void) outToProcessFile:(NSData *)data {
    if (data) {
        NSFileHandle *fileHandle = [NSFileHandle fileHandleForUpdatingAtPath:_processResfile];
        [fileHandle seekToEndOfFile];
        [fileHandle writeData:data];
        [fileHandle closeFile];
    }
}

- (void)printResult:(NSString *)res {
    self.ttsTextView.text = [[NSString stringWithFormat:@"%@\r\n",res] stringByAppendingString:self.ttsTextView.text];
    //self.ttsTextView.text = res;
    [self.ttsTextView scrollRangeToVisible:NSMakeRange(0, 0)];
}

- (void)printLogs:(NSString *)log {
    self.logsTextView.text = [[NSString stringWithFormat:@"%@ %@\r\n",[self currentDateStr], log] stringByAppendingString:self.logsTextView.text];
    //self.logsTextView.text = log;
    [self.logsTextView scrollRangeToVisible:NSMakeRange(0, 0)];
}

//获取当前时间
- (NSString *)currentDateStr {
    NSDate *currentDate = [NSDate date];//获取当前时间，日期
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];// 创建一个时间格式化对象
    [dateFormatter setDateFormat:@"YYYY/MM/dd HH:mm:ss"];//设定时间格式,这里可以设置成自己需要的格式
    NSString *dateString = [dateFormatter stringFromDate:currentDate];//将时间转化成字符串
    return dateString;
}

#pragma mark --configureations

- (void) setConfigs {
    [self userConfigs];
    TTSConfig *instance = [TTSConfig sharedInstance];
    if (instance == nil) {
        return;
    }
    if (_speechSynthesizer == nil) {
        return;
    }
    
    NSInteger sampleRateVal = [[_speechSynthesizer parameterForKey:JDS_TTS_SAMPLE_RATE] integerValue];
    if (instance.sampleRateVal != sampleRateVal) {
        [_speechSynthesizer setParameter:@(instance.sampleRateVal) forKey:JDS_TTS_SAMPLE_RATE];
    }
    NSString *appIDVal = [_speechSynthesizer parameterForKey:JDS_TTS_APP_ID];
    if (![instance.appIDVal isEqualToString:appIDVal]) {
        [_speechSynthesizer setParameter:instance.appIDVal forKey:JDS_TTS_APP_ID];
    }
    NSString *appKeyVal = [_speechSynthesizer parameterForKey:JDS_TTS_APP_KEY];
    if (![instance.appKeyVal isEqualToString:appKeyVal]) {
        [_speechSynthesizer setParameter:instance.appKeyVal forKey:JDS_TTS_APP_KEY];
    }
    NSString *secretKeyVal = [_speechSynthesizer parameterForKey:JDS_TTS_SECRET_KEY];
    if (![instance.secretKeyVal isEqualToString:secretKeyVal]) {
        [_speechSynthesizer setParameter:instance.secretKeyVal forKey:JDS_TTS_SECRET_KEY];
    }
    NSString *serverURLVal = [_speechSynthesizer parameterForKey:JDS_TTS_SERVER_URL];
    if (![instance.serverURLVal isEqualToString:serverURLVal]) {
        [_speechSynthesizer setParameter:instance.serverURLVal forKey:JDS_TTS_SERVER_URL];
    }
    NSInteger languageVal = [[_speechSynthesizer parameterForKey:JDS_TTS_LANGUAGE] integerValue];
    if (instance.languageVal != languageVal) {
        [_speechSynthesizer setParameter:@(instance.languageVal) forKey:JDS_TTS_LANGUAGE];
    }
    NSInteger timberVal = [[_speechSynthesizer parameterForKey:JDS_TTS_TIM_TYPE] integerValue];
    if (instance.timberVal != timberVal) {
        [_speechSynthesizer setParameter:@(instance.timberVal) forKey:JDS_TTS_TIM_TYPE];
    }
    NSInteger aueVal = [[_speechSynthesizer parameterForKey:JDS_TTS_AUE_TYPE] integerValue];
    if (instance.aueVal != aueVal) {
        [_speechSynthesizer setParameter:@(instance.aueVal) forKey:JDS_TTS_AUE_TYPE];
    }
    float volumeVal = [[_speechSynthesizer parameterForKey:JDS_TTS_VOLUME_LEVEL] floatValue];
    if (instance.volumeVal != volumeVal) {
        [_speechSynthesizer setParameter:@(instance.volumeVal) forKey:JDS_TTS_VOLUME_LEVEL];
    }
    float speedVal = [[_speechSynthesizer parameterForKey:JDS_TTS_SPEED_LEVEL] floatValue];
    if (instance.speedVal != speedVal) {
        [_speechSynthesizer setParameter:@(instance.speedVal) forKey:JDS_TTS_SPEED_LEVEL];
    }
    NSInteger timeStampVal = [[_speechSynthesizer parameterForKey:JDS_TTS_TIME_STAMP] integerValue];
    if (instance.timeStampVal != timeStampVal) {
        [_speechSynthesizer setParameter:@(instance.timeStampVal) forKey:JDS_TTS_TIME_STAMP];
    }
    
    NSInteger engineType = [[_speechSynthesizer parameterForKey:JDS_TTS_ENGINE_TYPE] integerValue];
    if (instance.ttsEngineType != engineType) {
        [_speechSynthesizer setParameter:@(instance.ttsEngineType) forKey:JDS_TTS_ENGINE_TYPE];
    }
    if (engineType == EJDS_TTSLocal) {
        /*NSString *feModel = [CommonUtil documentsPath:@"jd_tts_text.dat"];
        NSString *tts_res_path = [_speechSynthesizer parameterForKey:JDS_TTS_RES_PATH];
        
        NSString *model   = [CommonUtil documentsPath:@"taotao.dat"];
        NSString *tts_voice_path = [_speechSynthesizer parameterForKey:JDS_TTS_VOICE_PATH];
        
        if (![feModel isEqualToString:tts_res_path]) {
            [_speechSynthesizer setParameter:feModel forKey:JDS_TTS_RES_PATH];
        }
        if (![model isEqualToString:tts_voice_path]) {
            [_speechSynthesizer setParameter:model forKey:JDS_TTS_VOICE_PATH];
        }*/
    }
}

- (void) userConfigs {
    TTSConfig *instance = [TTSConfig sharedInstance];
    if (instance == nil) {
        return;
    }
 
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    
    if([userDefaults objectForKey:JDS_TTS_SAMPLE_RATE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_SAMPLE_RATE];
        if (index == 0)
            instance.sampleRateVal = EJDS_TTSSampleRate8K;
        else if (index == 1)
            instance.sampleRateVal = EJDS_TTSSampleRate12K;
        else if (index == 2)
            instance.sampleRateVal = EJDS_TTSSampleRate16K;
        else if (index == 3)
            instance.sampleRateVal = EJDS_TTSSampleRate24K;
        else
            instance.sampleRateVal = EJDS_TTSSampleRate24K;
    } else {
        instance.sampleRateVal = kSampleRateDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_APP_ID]) {
        instance.appIDVal = [userDefaults stringForKey:JDS_TTS_APP_ID];
    } else {
        instance.appIDVal = kApplicationIdDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_APP_KEY]) {
        instance.appKeyVal = [userDefaults stringForKey:JDS_TTS_APP_KEY];
    } else {
        instance.appKeyVal = kApplicationKeyDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SECRET_KEY]) {
        instance.secretKeyVal = [userDefaults stringForKey:JDS_TTS_SECRET_KEY];
    } else {
        instance.secretKeyVal = kSecretKeyDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SERVER_URL]) {
        instance.serverURLVal = [userDefaults stringForKey:JDS_TTS_SERVER_URL];
    } else {
        instance.serverURLVal = kServerURLDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_LANGUAGE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_LANGUAGE];
        if (index == 0)
            instance.languageVal = EJDS_TTSLanguageMandarin;
        else if (index == 1)
            instance.languageVal = EJDS_TTSLanguageCantonese;
        else if (index == 2)
            instance.languageVal = EJDS_TTSLanguageEnglish;
        else
            instance.languageVal = EJDS_TTSLanguageMandarin;
    } else {
        instance.languageVal = kLanguageDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_TIM_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_TIM_TYPE];
        if (index == 0)
            instance.timberVal = EJDS_TTS_TAOTAO_FEMALE;
        else if (index == 1)
            instance.timberVal = EJDS_TTS_BINBIN_MALE;
        else if (index == 2)
            instance.timberVal = EJDS_TTS_TINGTING_FEMALE;
        else if (index == 3)
            instance.timberVal = EJDS_TTS_GUOJING_MALE;
        else if (index == 4)
            instance.timberVal = EJDS_TTS_MONICA_FEMALE;
        else if (index == 5)
            instance.timberVal = EJDS_TTS_DELORES_FEMALE;
        else
            instance.timberVal = EJDS_TTS_TAOTAO_FEMALE;
    } else {
        instance.timberVal = kTimDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_AUE_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_AUE_TYPE];
        if (index == 0)
            instance.aueVal = EJDS_TTSAudioFormatWAV;
        else if (index == 1)
            instance.aueVal = EJDS_TTSAudioFormatPCM;
        else
            instance.aueVal = EJDS_TTSAudioFormatOPUS;
    } else {
        instance.aueVal = kAueDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_VOLUME_LEVEL]) {
        float val = [userDefaults floatForKey:JDS_TTS_VOLUME_LEVEL];
        if (val < 0.1f || val > 10.0f )
            instance.volumeVal = kVolumeLevelDef;
        else
            instance.volumeVal = val;
    } else {
        instance.volumeVal = kVolumeLevelDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SPEED_LEVEL]) {
        float val = [userDefaults floatForKey:JDS_TTS_SPEED_LEVEL];
        if (val < 0.1f || val > 10.0f )
            instance.speedVal = kSpeedLevelDef;
        else
            instance.speedVal = val;
    } else {
        instance.speedVal = kSpeedLevelDef;
    }

    if([userDefaults objectForKey:JDS_TTS_TIME_STAMP]) {
        BOOL val = [userDefaults floatForKey:JDS_TTS_TIME_STAMP];
        instance.timeStampVal = val ? 1 : 0;
    } else {
        instance.timeStampVal = kTimeStampDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_TASK_TYPE]) {
        NSInteger val = [userDefaults floatForKey:JDS_TTS_TASK_TYPE];
        instance.taskTypeVal = val;
    } else {
        instance.taskTypeVal = kTaskDef;
    }
    
    if([userDefaults objectForKey:kIsSaveFileKey]) {
        NSInteger val = [userDefaults floatForKey:kIsSaveFileKey];
        instance.saveFileVal = val;
    } else {
        instance.saveFileVal = kIsSaveFileDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_ENGINE_TYPE]) {
        NSInteger val = [userDefaults floatForKey:JDS_TTS_ENGINE_TYPE];
        instance.ttsEngineType = val;
    } else {
        instance.ttsEngineType = kEngineDef;
    }

#ifdef __TEST
    // Testing
    if ([instance.appIDVal isEqualToString:@""]) {
        instance.appIDVal = kApplicationIdTestDef;
    }
    if ([instance.serverURLVal isEqualToString:@""]) {
        instance.serverURLVal = kServerURLTestDef;
    }
    if ([instance.appKeyVal isEqualToString:@""]) {
        instance.appKeyVal = kApplicationKeyTestDef;
    }
    if ([instance.secretKeyVal isEqualToString:@""]) {
        instance.secretKeyVal = kSecretKeyTestDef;
    }
#endif
}


#pragma mark --Settings

// 更改配置并且存储到默认设置

- (IBAction)sampleRateDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.sampleRateChoice.selectedSegmentIndex forKey:JDS_TTS_SAMPLE_RATE];
    [userDefaults synchronize];
}

- (IBAction)APPIDEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.APPID.text forKey:JDS_TTS_APP_ID];
    [userDefaults synchronize];
}

- (IBAction)APPKeyEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.AppKey.text forKey:JDS_TTS_APP_KEY];
    [userDefaults synchronize];
}

- (IBAction)SecretKeyEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.SecretKey.text forKey:JDS_TTS_SECRET_KEY];
    [userDefaults synchronize];
}

- (IBAction)serverURLEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.serverURL.text forKey:JDS_TTS_SERVER_URL];
    [userDefaults synchronize];
}

- (IBAction)languageDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.language.selectedSegmentIndex forKey:JDS_TTS_LANGUAGE];
    [userDefaults synchronize];
}

- (IBAction)timbreDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.timbre.selectedSegmentIndex forKey:JDS_TTS_TIM_TYPE];
    [userDefaults synchronize];
}

- (IBAction)aueDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.aueChoice.selectedSegmentIndex forKey:JDS_TTS_AUE_TYPE];
    [userDefaults synchronize];
}

- (IBAction)setVolume:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setFloat:self.volumeSlider.value forKey:JDS_TTS_VOLUME_LEVEL];
    self.volumeRes.text = [NSString stringWithFormat:@"%.1f", self.volumeSlider.value];
    [userDefaults synchronize];
}

- (IBAction)setSpeed:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setFloat:self.speedSlider.value forKey:JDS_TTS_SPEED_LEVEL];
    self.speedRes.text = [NSString stringWithFormat:@"%.1f", self.speedSlider.value];
    [userDefaults synchronize];
}

- (IBAction)timeStampSwitch:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.timeStampSwitch.on forKey:JDS_TTS_TIME_STAMP];
    [userDefaults synchronize];
}

- (IBAction)engineChioce:(id)sender {
}

- (IBAction)taskChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.taskChoice.selectedSegmentIndex forKey:JDS_TTS_TASK_TYPE];
    [userDefaults synchronize];
}

- (IBAction)toggleSaveFile:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.isSaveFile.on forKey:kIsSaveFileKey];
    [userDefaults synchronize];
}

- (IBAction)engineChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.engineType.selectedSegmentIndex forKey:JDS_TTS_ENGINE_TYPE];
    [userDefaults synchronize];
}

- (IBAction)defaultSet:(id)sender {
    [self mySetDefault];
}

// 恢复到默认设置
- (void)mySetDefault {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];

    self.sampleRateChoice.selectedSegmentIndex = 3;
    [userDefaults setInteger:self.sampleRateChoice.selectedSegmentIndex forKey:JDS_TTS_SAMPLE_RATE];
    
    self.APPID.text = kApplicationIdDef;
    [userDefaults setObject:self.APPID.text forKey:JDS_TTS_APP_ID];
    
    self.AppKey.text = kApplicationKeyDef;
    [userDefaults setObject:self.AppKey.text forKey:JDS_TTS_APP_KEY];
    
    self.SecretKey.text = kSecretKeyDef;
    [userDefaults setObject:self.SecretKey.text forKey:JDS_TTS_SECRET_KEY];
    
    self.serverURL.text = kServerURLDef;
    [userDefaults setObject:self.serverURL.text forKey:JDS_TTS_SERVER_URL];

    self.language.selectedSegmentIndex = 0;
    [userDefaults setInteger:self.language.selectedSegmentIndex forKey:JDS_TTS_LANGUAGE];
    
    self.timbre.selectedSegmentIndex = kTimDef;
    [userDefaults setInteger:self.timbre.selectedSegmentIndex forKey:JDS_TTS_TIM_TYPE];
    
    self.aueChoice.selectedSegmentIndex = kAueDef;
    [userDefaults setInteger:self.aueChoice.selectedSegmentIndex forKey:JDS_TTS_AUE_TYPE];
    
    self.volumeSlider.value = kVolumeLevelDef;
    [userDefaults setInteger:self.volumeSlider.value forKey:JDS_TTS_VOLUME_LEVEL];
    self.volumeRes.text = [NSString stringWithFormat:@"%.1f", self.volumeSlider.value];
    
    self.speedSlider.value = kSpeedLevelDef;
    [userDefaults setInteger:self.speedSlider.value forKey:JDS_TTS_SPEED_LEVEL];
    self.speedRes.text = [NSString stringWithFormat:@"%.1f", self.speedSlider.value];
    
    self.timeStampSwitch.on = (BOOL)kTimeStampDef;
    [userDefaults setBool:self.timeStampSwitch.on forKey:JDS_TTS_TIME_STAMP];
    
    self.taskChoice.selectedSegmentIndex = kTaskDef;
    [userDefaults setInteger:self.taskChoice.selectedSegmentIndex forKey:JDS_TTS_TASK_TYPE];
    
    self.isSaveFile.on = kIsSaveFileDef;
    [userDefaults setBool:self.isSaveFile.on forKey:kIsSaveFileKey];

    self.engineType.selectedSegmentIndex = kEngineDef;
    [userDefaults setInteger:self.engineType.selectedSegmentIndex forKey:JDS_TTS_ENGINE_TYPE];
    
    [userDefaults synchronize];
}

// 加载默认设置选项
- (void)myViewLoad {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    
    if([userDefaults objectForKey:JDS_TTS_SERVER_URL]) {
        self.serverURL.text = [userDefaults stringForKey:JDS_TTS_SERVER_URL];
    } else {
        self.serverURL.text = kServerURLDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_APP_ID]) {
        self.APPID.text = [userDefaults stringForKey:JDS_TTS_APP_ID];
    } else {
        self.APPID.text = kApplicationIdDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_APP_KEY]) {
        self.AppKey.text = [userDefaults stringForKey:JDS_TTS_APP_KEY];
    } else {
        self.AppKey.text = kApplicationKeyDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SECRET_KEY]) {
        self.SecretKey.text = [userDefaults stringForKey:JDS_TTS_SECRET_KEY];
    } else {
        self.SecretKey.text = kSecretKeyDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SAMPLE_RATE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_SAMPLE_RATE];
        [self.sampleRateChoice setSelectedSegmentIndex:index];
    } else {
        [self.sampleRateChoice setSelectedSegmentIndex:3];
    }
    
    if([userDefaults objectForKey:JDS_TTS_LANGUAGE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_LANGUAGE];
        [self.language setSelectedSegmentIndex:index];
    } else {
        [self.language setSelectedSegmentIndex:0];
    }
    
    if([userDefaults objectForKey:JDS_TTS_TIM_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_TIM_TYPE];
        [self.timbre setSelectedSegmentIndex:index];
    } else {
        [self.timbre setSelectedSegmentIndex:kTimDef];
    }
    
    if([userDefaults objectForKey:JDS_TTS_AUE_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_AUE_TYPE];
        [self.aueChoice setSelectedSegmentIndex:index];
    } else {
        [self.aueChoice setSelectedSegmentIndex:kAueDef];
    }
    
    if([userDefaults objectForKey:JDS_TTS_VOLUME_LEVEL]) {
        float val = [userDefaults floatForKey:JDS_TTS_VOLUME_LEVEL];
        self.volumeSlider.value = val;
    } else {
        self.volumeSlider.value = kVolumeLevelDef;
    }
    self.volumeRes.text = [NSString stringWithFormat:@"%.1f", self.volumeSlider.value];
    
    if([userDefaults objectForKey:JDS_TTS_SPEED_LEVEL]) {
        float val = [userDefaults floatForKey:JDS_TTS_SPEED_LEVEL];
        self.speedSlider.value = val;
    } else {
        self.speedSlider.value = kSpeedLevelDef;
    }
    self.speedRes.text = [NSString stringWithFormat:@"%.1f", self.speedSlider.value];
    
    if([userDefaults objectForKey:JDS_TTS_TIME_STAMP]) {
        BOOL val = [userDefaults floatForKey:JDS_TTS_TIME_STAMP];
        self.timeStampSwitch.on = val;
    } else {
        self.timeStampSwitch.on = (BOOL)kTimeStampDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_TASK_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_TASK_TYPE];
        [self.taskChoice setSelectedSegmentIndex:index];
    } else {
        [self.taskChoice setSelectedSegmentIndex:kTaskDef];
    }

    if([userDefaults objectForKey:kIsSaveFileKey]) {
        BOOL val = [userDefaults floatForKey:kIsSaveFileKey];
        self.isSaveFile.on = val;
    } else {
        self.isSaveFile.on = kIsSaveFileDef;
    }

    if([userDefaults objectForKey:JDS_TTS_ENGINE_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_ENGINE_TYPE];
        [self.engineType setSelectedSegmentIndex:index];
    } else {
        [self.engineType setSelectedSegmentIndex:kEngineDef];
    }
}

#pragma mark - write to file

#ifdef __SAVE_TO_FILE
- (void) creatOutFile
{
    // 获取Documents路径
    NSArray *path = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [path objectAtIndex:0];
    
    // 获取文件路径
    NSTimeInterval timestamp = [NSDate timeIntervalSinceReferenceDate];
    _outfile = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"ttsAudio-%f.pcm",timestamp]];
    NSLog(@"Output file:%@", _outfile);
    
    // 如果文件不存在
    if (![[NSFileManager defaultManager] fileExistsAtPath:_outfile])
    {
        [[NSFileManager defaultManager] createFileAtPath:_outfile contents:nil attributes:nil];
    }
}

- (void) outToFile:(NSData *)data
{
    if (data) {
        NSFileHandle *fileHandle = [NSFileHandle fileHandleForUpdatingAtPath:_outfile];
        [fileHandle seekToEndOfFile];
        [fileHandle writeData:data];
        [fileHandle closeFile];
    }
}
#endif

#pragma mark - functions

// Dictionary to JSON
+ (NSString *)convertToJsonData:(NSDictionary *)dict {
    
    NSError *error;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:dict options:NSJSONWritingPrettyPrinted error:&error];
    
    NSString *jsonString;
    if (!jsonData) {
        NSLog(@"%@",error);
        return nil;
    }else{
        jsonString = [[NSString alloc]initWithData:jsonData encoding:NSUTF8StringEncoding];
    }
    
    return jsonString;
}

// JSON to dictionary
+ (NSDictionary *)dictionaryWithJsonString:(NSString *)jsonString {
    if (jsonString == nil) {
        return nil;
    }
    
    NSData *jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    NSError *err;
    NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:jsonData
                                                        options:NSJSONReadingMutableContainers
                                                          error:&err];
    if(err)
    {
        NSLog(@"json解析失败：%@",err);
        return nil;
    }
    return dic;
}

@end
