//
//  JDViewController.m
//  JD_Speech_Test
//
//  Created by 满达 on 2018/6/9.
//  Copyright © 2018年 满达. All rights reserved.
//

#import "JDViewController.h"
#import <JDSpeechSynthesizer/JDSpeechSynthesizer.h>
#import <JDSpeechSynthesizer/JDSpeechSynthesizerDelegate.h>
#import <JDSpeechSynthesizer/JDSTTSDefines.h>
#import <JDSpeechSynthesizer/JDSTTSParameters.h>
#import <JDSpeechSynthesizer/JDSpeechError.h>
#import "TTSConfig.h"
#import "CommonUtil.h"

#pragma mark - defines

// Default values
static NSString *const kTextDef                = @"你好,京东！";
static const NSInteger kSampleRateDef          = EJDS_TTSSampleRate24K;
static const float     kVolumeLevelDef         = 2.0f;
static const float     kSpeedLevelDef          = 1.0f;
static NSString *const kApplicationIdDef       = @"";
static NSString *const kSecretKeyDef           = @"";
static NSString *const kApplicationKeyDef      = @"";
static NSString *const kServerURLDef           = @"";
static const NSInteger kLanguageDef            = EJDS_TTSLanguageMandarin;
static const NSInteger kTimDef                 = EJDS_TTS_TAOTAO_FEMALE;
static const NSInteger kAueDef                 = EJDS_TTSAudioFormatOPUS;
static const NSInteger kTimeStampDef           = ETTSTimeStampEna;
static const NSInteger kTaskDef                = 0; // 0：播报 1：合成
static const NSInteger kEngineDef              = 0; // 0：离线 1：在线

static NSString *const JDS_TTS_TASK_TYPE       = @"TTS.TaskType";
static NSString *const kFEModelName            = @"jd_tts_text.dat";
static NSString *const kModelName              = @"binbin.dat";

// 可以查看此机制，实现多次连续合成t调用
static const NSInteger kSynTimes               = 1;

#define __TEST
//#undef __TEST

#ifdef __TEST
// Testing
// APP ID调用NeuHub平台无需设置，具体特殊需求发送到aitts-contact@jd.com进行申请
static NSString *const kApplicationIdTestDef   = @"7a12e9ac9a82670a2fe986bf5204d1**";
static NSString *const kApplicationKeyTestDef  = @"894866b6ce511b438a974bdaa98b61**";
static NSString *const kSecretKeyTestDef       = @"a6eba1a3cedf62cf0347d644062471**";
static NSString *const kServerURLTestDef       = @"https://aiapi.jd.com/jdai/tts";
// 普通音色 https://aiapi.jd.com/jdai/tts
// 精品音色 https://aiapi.jd.com/jdai/tts_vip
// 虚拟主播 https://aiapi.jd.com/jdai/tts_virtual
// NeuHub接口 http://tts-stream.jd.com/tts
#endif

//#define __SAVE_TO_FILE
#undef __SAVE_TO_FILE

@interface JDViewController () <JDSpeechSynthesizerDelegate> {
    BOOL                  _isRunning;
    BOOL                  _isPlaying;
    BOOL                  _isPaused;
    
    NSInteger             _taskCount;   // 目前运行任务数
    NSInteger             _startCount;  // 开始提交任务数
    NSInteger             _synPlayIndex;// 播放还是合成任务类型
    NSInteger             _taskTimes;   // 预定合成任务数剩余多少
    NSInteger             _playTimes;   // 预定播放任务数剩余多少
    
    // 离线引擎相关的前端资源文件以及模型文件
    __block BOOL                  _femodelLoaded;
    __block BOOL                  _modelLoaded;
}

// Outlets
@property (weak, nonatomic) IBOutlet UITextView *ttsTextView;
@property (weak, nonatomic) IBOutlet UITextView *logsTextView;

@property (weak, nonatomic) IBOutlet UISegmentedControl *sampleRateChoice;
@property (weak, nonatomic) IBOutlet UITextField *APPID;
@property (weak, nonatomic) IBOutlet UITextField *AppKey;
@property (weak, nonatomic) IBOutlet UITextField *SecretKey;
@property (weak, nonatomic) IBOutlet UITextField *serverURL;
@property (weak, nonatomic) IBOutlet UISegmentedControl *language;
@property (weak, nonatomic) IBOutlet UISegmentedControl *timbre;
@property (weak, nonatomic) IBOutlet UISegmentedControl *aueChoice;
@property (weak, nonatomic) IBOutlet UISlider *volumeSlider;
@property (weak, nonatomic) IBOutlet UISlider *speedSlider;
@property (weak, nonatomic) IBOutlet UILabel *volumeRes;
@property (weak, nonatomic) IBOutlet UILabel *speedRes;
@property (weak, nonatomic) IBOutlet UISwitch *timeStampSwitch;
@property (weak, nonatomic) IBOutlet UISegmentedControl *taskChoice;
@property (weak, nonatomic) IBOutlet UISegmentedControl *engineType;

// Actions
- (IBAction)sampleRateDidChoice:(id)sender;
- (IBAction)APPIDEdit:(id)sender;
- (IBAction)APPKeyEdit:(id)sender;
- (IBAction)SecretKeyEdit:(id)sender;
- (IBAction)serverURLEdit:(id)sender;
- (IBAction)languageDidChoice:(id)sender;
- (IBAction)timbreDidChoice:(id)sender;
- (IBAction)aueDidChoice:(id)sender;
- (IBAction)setVolume:(id)sender;
- (IBAction)setSpeed:(id)sender;
- (IBAction)timeStampSwitch:(id)sender;
- (IBAction)taskChoice:(id)sender;
- (IBAction)engineChoice:(id)sender;

- (IBAction)defaultSet:(id)sender;

//-----------------------------------------------------------------------------------
@property (nonatomic,strong) JDSpeechSynthesizer  *speechSynthesizer; 

@property (weak, nonatomic) IBOutlet UIButton *startBtn;
@property (weak, nonatomic) IBOutlet UIButton *stopBtn;
@property (weak, nonatomic) IBOutlet UIButton *pauseResumeBtn;
@property (weak, nonatomic) IBOutlet UIButton *loadModel;

#ifdef __SAVE_TO_FILE
@property(atomic,strong)  NSString      *outfile;
#endif

@property(nonatomic, strong) dispatch_queue_t queueTask;

@property(atomic)     NSTimeInterval timestampStart;
@property(atomic)     NSTimeInterval timestampResponse;
@property(atomic)     NSTimeInterval timestampEnd;

- (void)mySetDefault;
- (void)myViewLoad;

@end

@implementation JDViewController

#pragma mark --defaultMethods

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    // 启动程序的时候预先加载的设置值
    [self myViewLoad];
    /*if (!_speechSynthesizer) {
        // 涉及到前端资源以及离线模型加载，可以另外以其他异步机制加载
        _speechSynthesizer = [self speechSynthesizer];
    }*/
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    //NSLog(@"viewWillAppear");
    [self initializeBtn];
    _isRunning = NO;
    _isPlaying = NO;
    _isPaused  = NO;
}

- (void) viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    
    //NSLog(@"viewWillDisappear");
    if (_speechSynthesizer) {
        // 如果后面继续使用stop即可
        // NSLog(@"will stop");
        //[_speechSynthesizer stop];
        // 如果不用合成对象销毁即可
        NSLog(@"will destroy");
        [JDSpeechSynthesizer destroy];
        _speechSynthesizer = nil;
    }
}

#pragma mark - Initialization

- (JDSpeechSynthesizer *) speechSynthesizer {
    //TTS singleton
    if (_speechSynthesizer == nil) {
        _speechSynthesizer = [JDSpeechSynthesizer sharedInstance];
        //[self printLogs:[NSString stringWithFormat:@"TTS版本：%@", JDSpeechSynthesizerVersion]];
        _speechSynthesizer.delegate = self;
    }
    
    // 可以参考JDSTTSParameters.h进行定义
    // 没有更改的参数项，无需设置
    [self setConfigs];
    
    return _speechSynthesizer;
}

#pragma mark --Operations

- (void) initializeBtn {
    self.startBtn.enabled       = YES;
    self.pauseResumeBtn.enabled = NO;
    self.stopBtn.enabled        = NO;
    
    _isPaused                   = NO;
    [self.pauseResumeBtn setTitle:@"||" forState:UIControlStateNormal];
}

- (void) startBtnTouch {
    self.startBtn.enabled       = NO;
    self.pauseResumeBtn.enabled = NO;
    self.stopBtn.enabled        = YES;
    
    _isPaused                   = NO;
}

- (void) stopBtnTouch {
    self.startBtn.enabled       = NO;
    self.pauseResumeBtn.enabled = NO;
    self.stopBtn.enabled        = NO;
    
    _isPaused                   = NO;
    [self.pauseResumeBtn setTitle:@"||" forState:UIControlStateNormal];
}

- (void) pauseResumeBtnTouch {
    if (self.pauseResumeBtn.enabled && _isPaused) {
        _isPaused = NO;
        [self.pauseResumeBtn setTitle:@"||" forState:UIControlStateNormal];
        [self.speechSynthesizer resumeSpeaking];
    } else if (self.pauseResumeBtn.enabled && !_isPaused) {
        _isPaused = YES;
        [self.pauseResumeBtn setTitle:@">>" forState:UIControlStateNormal];
        [self.speechSynthesizer pauseSpeaking];
    }
}

- (IBAction)start:(id)sender {
    [self userConfigs];
    TTSConfig *instance = [TTSConfig sharedInstance];
    if (instance.ttsEngineType == EJDS_TTSLocal) {
        if (!_femodelLoaded || !_modelLoaded) {
            NSLog(@"haven't loaded model");
            [self printLogs:@"haven't loaded model"];
            return;
        }
    }
    if (_isRunning) {
        NSLog(@"haven't finished");
        [self printLogs:@"haven't finished"];
        return;
    }
    if ([self.ttsTextView.text isEqualToString:@""]) {
        self.ttsTextView.text = kTextDef;
    }
    
    NSLog(@">>start");
    self.logsTextView.text = @">>start";
    [self startBtnTouch];
    _isRunning = YES;
    _isPlaying = NO;
    
#ifdef __SAVE_TO_FILE
    // 创建一个file
    [self creatOutFile];
#endif
    
    _taskCount  = 0;
    _startCount = 0;
    _taskTimes  = kSynTimes;
    _playTimes  = kSynTimes;
    self.queueTask = nil;
    self.queueTask = dispatch_queue_create("com.jd.JDSpeechSynthesizer.Test", NULL);
    [self runTask];
}

- (IBAction)stop:(id)sender {
    NSLog(@">>stop");
    self.logsTextView.text = @">>stop";
    
    [self stopBtnTouch];
    [self.speechSynthesizer stop];
    
    _isRunning = NO;
    _isPlaying = NO;
    _taskCount  = 0;
    _startCount = 0;
    _taskTimes  = 0;
    _playTimes  = 0;
    [self initializeBtn];
}

- (IBAction)pauseResume:(id)sender {
    NSLog(@">>pauseResume");
    self.logsTextView.text = @">>pauseResume";
    [self pauseResumeBtnTouch];
}

- (IBAction)loadModel:(id)sender {
    [self userConfigs];
    TTSConfig *instance = [TTSConfig sharedInstance];
    if (instance.ttsEngineType != EJDS_TTSLocal) {
        return;
    }
    if (!_speechSynthesizer) {
        // 涉及到前端资源以及离线模型加载，可以另外以其他异步机制加载
        _speechSynthesizer = [JDSpeechSynthesizer sharedInstance];
        //[self printLogs:[NSString stringWithFormat:@"TTS版本：%@", JDSpeechSynthesizerVersion]];
        _speechSynthesizer.delegate = self;
    }
    NSString *feModel = [CommonUtil documentsPath:kFEModelName];
    NSString *tts_res_path = [_speechSynthesizer parameterForKey:JDS_TTS_RES_PATH];
    NSString *model   = [CommonUtil documentsPath:kModelName];
    NSString *tts_voice_path = [_speechSynthesizer parameterForKey:JDS_TTS_VOICE_PATH];
    
    if (![CommonUtil fileExist:feModel]) {
        NSLog(@"No exist:%@", feModel);
        [self printLogs:[NSString stringWithFormat:@"No exist:%@", feModel]];
    }
    
    if (![CommonUtil fileExist:model]) {
        NSLog(@"No exist:%@", model);
        [self printLogs:[NSString stringWithFormat:@"No exist:%@", model]];
    }

    if ([CommonUtil fileExist:feModel] && ![feModel isEqualToString:tts_res_path]) {
        [_speechSynthesizer setParameter:feModel forKey:JDS_TTS_RES_PATH];
    }
    if ([CommonUtil fileExist:model] && ![model isEqualToString:tts_voice_path]) {
        [_speechSynthesizer setParameter:model forKey:JDS_TTS_VOICE_PATH];
    }
}

- (void)printLogs:(NSString *)log {
    self.logsTextView.text = [[NSString stringWithFormat:@"%@\r\n",log] stringByAppendingString:self.logsTextView.text];
    [self.logsTextView scrollRangeToVisible:NSMakeRange(0, 0)];
}

- (void)runTask{

    self.logsTextView.text = @"";
    __weak typeof(self) weakSelf = self;
    dispatch_sync(self.queueTask, ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf->_isRunning) {
            // 可以参考JDSTTSParameters.h进行定义
            // 没有更改的参数项，无需设置
            [strongSelf setConfigs];
            
            NSString *text = [NSString stringWithString:self.ttsTextView.text];
            strongSelf.timestampStart = [NSDate timeIntervalSinceReferenceDate];
            NSLog(@"[%f]Start run:%@", strongSelf.timestampStart, text);
            [strongSelf printLogs:[NSString stringWithFormat:@"[%f]Start run:%@",strongSelf.timestampStart, text]];

            TTSConfig *instance = [TTSConfig sharedInstance];
            NSInteger index = kTaskDef;
            if (instance != nil) {
                index = instance.taskTypeVal;
            }
            _synPlayIndex = index;
            _taskCount++;
            if (_startCount < 3) {
                _startCount++;
                NSLog(@"Ryan start_count:%ld task_count:%ld", _startCount, _taskCount);
            }
            NSLog(@"Running the startSpeaking");
            if (index == 0) {
                [self.speechSynthesizer startSpeaking:text];
            } else {
                [self.speechSynthesizer synthesize:text];
            }
        }
    });
}

#pragma mark - implementation delegate

- (void) onCompleted:(JDSpeechError*) error {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<合成结束回调");
            [self printLogs:@"<<合成结束回调"];
            [self printLogs:[NSString stringWithFormat:@"结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc]];
            NSLog(@"%@",[NSString stringWithFormat:@"结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc]);
            strongSelf.timestampEnd = [NSDate timeIntervalSinceReferenceDate];
            NSLog(@"Duration time:%f", strongSelf.timestampEnd - strongSelf.timestampStart);
            [strongSelf printLogs:[NSString stringWithFormat:@"Duration time:%f", strongSelf.timestampEnd - strongSelf.timestampStart]];
            
            if (error.errorCode != ER_JDS_TTS_OK) {
                
                [strongSelf.speechSynthesizer stop];
                
                strongSelf->_taskCount  = 0;
                strongSelf->_startCount = 0;
                strongSelf->_taskTimes  = 0;
                strongSelf->_playTimes  = 0;
                strongSelf->_isRunning = NO;
                strongSelf->_isPlaying = NO;
                strongSelf->_pauseResumeBtn.enabled = NO;
                [strongSelf initializeBtn];
            } else {
                strongSelf->_taskTimes--;
                // 如果是合成任务
                if (strongSelf->_synPlayIndex == 1) {
                    strongSelf->_taskCount--;
                    NSLog(@"Ryan syn _taskCount:%ld", strongSelf->_taskCount);
                }
                if (strongSelf->_taskTimes > 0) {
                    if (error.errorCode != ER_JDS_TTS_CacheBufferLimit &&
                        error.errorCode != ER_JDS_TTS_SynRunning &&
                        strongSelf->_startCount < 3) {
                        // 开始阶段，低于缓冲上线，任务继续提交
                        NSLog(@"Start run1");
                        [strongSelf runTask];
                    } else if (error.errorCode == ER_JDS_TTS_CacheBufferLimit || error.errorCode == ER_JDS_TTS_SynRunning) {
                        NSLog(@"未播放缓冲达到上限或当前合成任务未完成");
                        [self printLogs:@"未播放缓冲达到上限或当前合成任务未完成"];
                    } else {
                        // 低于缓冲上线，任务继续提交
                        if (strongSelf->_synPlayIndex == 1 && strongSelf->_taskCount < 3) {
                            [strongSelf runTask];
                        }
                    }
                    
                } else {
                    if (strongSelf->_synPlayIndex == 1) {
                        [strongSelf.speechSynthesizer stop];
                        strongSelf->_isRunning = NO;
                        [strongSelf initializeBtn];
                    }
                }
            }
        }
    });
}

- (void) onBegin {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<开始合成回到");
            [strongSelf printLogs:@"<<开始合成回调"];
            NSTimeInterval timestamp = [NSDate timeIntervalSinceReferenceDate];
            [strongSelf printLogs:[NSString stringWithFormat:@"开始合成:%f", timestamp - strongSelf.timestampStart]];
        }
    });
}

- (void) onSynthesizeData:(NSData*) data timeStamp:(NSString *) timeStamp {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<合成语音数据回调");
            [strongSelf printLogs:@"<<合成语音数据回调"];
            if (timeStamp && (![timeStamp isKindOfClass:[NSNull class]]) && timeStamp.length > 0) {
                NSLog(@"timeStamp:%@", timeStamp);
                [strongSelf printLogs:[NSString stringWithFormat:@"timeStamp:%@", timeStamp]];
            }
            strongSelf.timestampResponse = [NSDate timeIntervalSinceReferenceDate];
            NSLog(@"Response time:%f", strongSelf.timestampResponse - strongSelf.timestampStart);
            [strongSelf printLogs:[NSString stringWithFormat:@"Response time:%f", strongSelf.timestampResponse - strongSelf.timestampStart]];
#ifdef __SAVE_TO_FILE
            // Write to file
            if (data && data.length > 0) {
                NSLog(@"data length: %ld", data.length);
                [strongSelf outToFile:data];
            }
#endif
        }
    });
}

- (void) onSynthesizeProgress:(float) progress message:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<合成进度回调");
            [strongSelf printLogs:@"<<合成进度回调"];
            [strongSelf printLogs:[NSString stringWithFormat:@"合成进度：%f",progress]];
            NSLog(@"合成进度：%f",progress);
        }
    });
}

- (void) onSpeakBegin {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<开始播报回调");
            [strongSelf printLogs:@"<<开始播报回调"];
            strongSelf->_isPlaying = YES;
            strongSelf->_pauseResumeBtn.enabled = YES;
        }
    });
}

- (void) onSpeakProgress:(float) progress message:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<播报进度回调");
            [self printLogs:@"<<播报进度回调"];
            [self printLogs:[NSString stringWithFormat:@"播报进度：%f",progress]];
            NSLog(@"播报进度：%f",progress);
        }
    });
}

- (void) onSpeakEnd {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<结束播报回调");
            [self printLogs:@"<<结束播报回调"];
            strongSelf->_playTimes--;
            strongSelf->_taskCount--;
            if (strongSelf->_playTimes > 0) {
                NSLog(@"Ryan play _taskCount:%ld", strongSelf->_taskCount);
                if (strongSelf->_startCount >= 3 && strongSelf->_taskTimes > 0 && strongSelf->_taskCount < 3) {
                    NSLog(@"Start run2");
                    [strongSelf runTask];
                }
            } else {
                [self.speechSynthesizer stop];
                strongSelf->_isRunning = NO;
                strongSelf->_isPlaying = NO;
                strongSelf->_pauseResumeBtn.enabled = NO;
                [strongSelf initializeBtn];
            }
        }
    });
}

- (void) onBufferStart:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<缓冲显示回调");
            [self printLogs:@"<<缓冲显示回调"];
        }
    });
}

- (void) onBufferEnd:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<缓冲隐藏回调");
            [self printLogs:@"<<缓冲隐藏回调"];
        }
    });
}

- (void) onSynthesizerCreated:(NSString *)msg {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            NSLog(@"<<离线语音语音合成实例创建完成");
            [self printLogs:@"<<离线语音语音合成实例创建完成"];
        }
    });
}


- (void) onParamSetted:(NSString*)key err:(JDSpeechError*) error {
    __weak typeof(self) weakSelf = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            [self printLogs:[NSString stringWithFormat:@"设置参数结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc]];
            NSLog(@"%@",[NSString stringWithFormat:@"设置参数结果:err_code:%ld, %@", (long)error.errorCode, error.errorDesc]);
            if (error.errorCode == 0 && [key isEqualToString:JDS_TTS_RES_PATH]) {
                strongSelf->_femodelLoaded = YES;
            }
            if (error.errorCode == 0 && [key isEqualToString:JDS_TTS_VOICE_PATH]) {
                strongSelf->_modelLoaded = YES;
            }
        }
    });
}


#pragma mark --configureations

- (void) setConfigs {
    NSLog(@"*-setConfigs enter-*");
    [self userConfigs];
    TTSConfig *instance = [TTSConfig sharedInstance];
    if (instance == nil) {
        NSLog(@"*-setConfigs exit1-*");
        return;
    }
    if (_speechSynthesizer == nil) {
        NSLog(@"*-setConfigs exit2-*");
        return;
    }
    
    NSInteger sampleRateVal = [[_speechSynthesizer parameterForKey:JDS_TTS_SAMPLE_RATE] integerValue];
    if (instance.sampleRateVal != sampleRateVal) {
        [_speechSynthesizer setParameter:@(instance.sampleRateVal) forKey:JDS_TTS_SAMPLE_RATE];
    }
    NSString *appIDVal = [_speechSynthesizer parameterForKey:JDS_TTS_APP_ID];
    if (![instance.appIDVal isEqualToString:appIDVal]) {
        [_speechSynthesizer setParameter:instance.appIDVal forKey:JDS_TTS_APP_ID];
    }
    NSString *appKeyVal = [_speechSynthesizer parameterForKey:JDS_TTS_APP_KEY];
    if (![instance.appKeyVal isEqualToString:appKeyVal]) {
        [_speechSynthesizer setParameter:instance.appKeyVal forKey:JDS_TTS_APP_KEY];
    }
    NSString *secretKeyVal = [_speechSynthesizer parameterForKey:JDS_TTS_SECRET_KEY];
    if (![instance.secretKeyVal isEqualToString:secretKeyVal]) {
        [_speechSynthesizer setParameter:instance.secretKeyVal forKey:JDS_TTS_SECRET_KEY];
    }
    NSString *serverURLVal = [_speechSynthesizer parameterForKey:JDS_TTS_SERVER_URL];
    if (![instance.serverURLVal isEqualToString:serverURLVal]) {
        [_speechSynthesizer setParameter:instance.serverURLVal forKey:JDS_TTS_SERVER_URL];
    }
    NSInteger languageVal = [[_speechSynthesizer parameterForKey:JDS_TTS_LANGUAGE] integerValue];
    if (instance.languageVal != languageVal) {
        [_speechSynthesizer setParameter:@(instance.languageVal) forKey:JDS_TTS_LANGUAGE];
    }
    NSInteger timberVal = [[_speechSynthesizer parameterForKey:JDS_TTS_TIM_TYPE] integerValue];
    if (instance.timberVal != timberVal) {
        [_speechSynthesizer setParameter:@(instance.timberVal) forKey:JDS_TTS_TIM_TYPE];
    }
    NSInteger aueVal = [[_speechSynthesizer parameterForKey:JDS_TTS_AUE_TYPE] integerValue];
    if (instance.aueVal != aueVal) {
        [_speechSynthesizer setParameter:@(instance.aueVal) forKey:JDS_TTS_AUE_TYPE];
    }
    float volumeVal = [[_speechSynthesizer parameterForKey:JDS_TTS_VOLUME_LEVEL] floatValue];
    if (instance.volumeVal != volumeVal) {
        [_speechSynthesizer setParameter:@(instance.volumeVal) forKey:JDS_TTS_VOLUME_LEVEL];
    }
    float speedVal = [[_speechSynthesizer parameterForKey:JDS_TTS_SPEED_LEVEL] floatValue];
    if (instance.speedVal != speedVal) {
        [_speechSynthesizer setParameter:@(instance.speedVal) forKey:JDS_TTS_SPEED_LEVEL];
    }
    NSInteger timeStampVal = [[_speechSynthesizer parameterForKey:JDS_TTS_TIME_STAMP] integerValue];
    if (instance.timeStampVal != timeStampVal) {
        [_speechSynthesizer setParameter:@(instance.timeStampVal) forKey:JDS_TTS_TIME_STAMP];
    }

    NSInteger engineType = [[_speechSynthesizer parameterForKey:JDS_TTS_ENGINE_TYPE] integerValue];
    if (instance.ttsEngineType != engineType) {
        [_speechSynthesizer setParameter:@(instance.ttsEngineType) forKey:JDS_TTS_ENGINE_TYPE];
    }
    if (engineType == EJDS_TTSLocal) {
        /*NSString *feModel = [CommonUtil documentsPath:@"jd_tts_text.dat"];
        NSString *tts_res_path = [_speechSynthesizer parameterForKey:JDS_TTS_RES_PATH];
        
        NSString *model   = [CommonUtil documentsPath:@"taotao.dat"];
        NSString *tts_voice_path = [_speechSynthesizer parameterForKey:JDS_TTS_VOICE_PATH];
        
        //NSString *dur_model = [CommonUtil documentsPath:@"duration.mlmodel"];
        //NSString *acou_model = [CommonUtil documentsPath:@"acoustic.mlmodel"];
        //NSLog(@"feModel:%@ model:%@ dur:%@  acou:%@", feModel, model, dur_model, acou_model);
        
        if (![feModel isEqualToString:tts_res_path]) {
            [_speechSynthesizer setParameter:feModel forKey:JDS_TTS_RES_PATH];
        }
        if (![model isEqualToString:tts_voice_path]) {
            [_speechSynthesizer setParameter:model forKey:JDS_TTS_VOICE_PATH];
        }*/
    }
    NSLog(@"*-setConfigs exit2-*");
}

- (void) userConfigs {
    //NSLog(@"*-userConfigs enter-*");
    TTSConfig *instance = [TTSConfig sharedInstance];
    if (instance == nil) {
        //NSLog(@"*-userConfigs exit1-*");
        return;
    }
 
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    
    if([userDefaults objectForKey:JDS_TTS_SAMPLE_RATE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_SAMPLE_RATE];
        if (index == 0)
            instance.sampleRateVal = EJDS_TTSSampleRate8K;
        else if (index == 1)
            instance.sampleRateVal = EJDS_TTSSampleRate12K;
        else if (index == 2)
            instance.sampleRateVal = EJDS_TTSSampleRate16K;
        else if (index == 3)
            instance.sampleRateVal = EJDS_TTSSampleRate24K;
        else
            instance.sampleRateVal = EJDS_TTSSampleRate24K;
    } else {
        instance.sampleRateVal = kSampleRateDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_APP_ID]) {
        instance.appIDVal = [userDefaults stringForKey:JDS_TTS_APP_ID];
    } else {
        instance.appIDVal = kApplicationIdDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_APP_KEY]) {
        instance.appKeyVal = [userDefaults stringForKey:JDS_TTS_APP_KEY];
    } else {
        instance.appKeyVal = kApplicationKeyDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SECRET_KEY]) {
        instance.secretKeyVal = [userDefaults stringForKey:JDS_TTS_SECRET_KEY];
    } else {
        instance.secretKeyVal = kSecretKeyDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SERVER_URL]) {
        instance.serverURLVal = [userDefaults stringForKey:JDS_TTS_SERVER_URL];
    } else {
        instance.serverURLVal = kServerURLDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_LANGUAGE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_LANGUAGE];
        if (index == 0)
            instance.languageVal = EJDS_TTSLanguageMandarin;
        else if (index == 1)
            instance.languageVal = EJDS_TTSLanguageCantonese;
        else if (index == 2)
            instance.languageVal = EJDS_TTSLanguageEnglish;
        else
            instance.languageVal = EJDS_TTSLanguageMandarin;
    } else {
        instance.languageVal = kLanguageDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_TIM_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_TIM_TYPE];
        if (index == 0)
            instance.timberVal = EJDS_TTS_TAOTAO_FEMALE;
        else if (index == 1)
            instance.timberVal = EJDS_TTS_BINBIN_MALE;
        else if (index == 2)
            instance.timberVal = EJDS_TTS_TINGTING_FEMALE;
        else if (index == 3)
            instance.timberVal = EJDS_TTS_GUOJING_MALE;
        else if (index == 4)
            instance.timberVal = EJDS_TTS_MONICA_FEMALE;
        else if (index == 5)
            instance.timberVal = EJDS_TTS_DELORES_FEMALE;
        else
            instance.timberVal = EJDS_TTS_TAOTAO_FEMALE;
    } else {
        instance.timberVal = kTimDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_AUE_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_AUE_TYPE];
        if (index == 0)
            instance.aueVal = EJDS_TTSAudioFormatWAV;
        else if (index == 1)
            instance.aueVal = EJDS_TTSAudioFormatPCM;
        else
            instance.aueVal = EJDS_TTSAudioFormatOPUS;
    } else {
        instance.aueVal = kAueDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_VOLUME_LEVEL]) {
        float val = [userDefaults floatForKey:JDS_TTS_VOLUME_LEVEL];
        if (val < 0.1f || val > 10.0f )
            instance.volumeVal = kVolumeLevelDef;
        else
            instance.volumeVal = val;
    } else {
        instance.volumeVal = kVolumeLevelDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SPEED_LEVEL]) {
        float val = [userDefaults floatForKey:JDS_TTS_SPEED_LEVEL];
        if (val < 0.1f || val > 10.0f )
            instance.speedVal = kSpeedLevelDef;
        else
            instance.speedVal = val;
    } else {
        instance.speedVal = kSpeedLevelDef;
    }

    if([userDefaults objectForKey:JDS_TTS_TIME_STAMP]) {
        BOOL val = [userDefaults floatForKey:JDS_TTS_TIME_STAMP];
        instance.timeStampVal = val ? 1 : 0;
    } else {
        instance.timeStampVal = kTimeStampDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_TASK_TYPE]) {
        NSInteger val = [userDefaults floatForKey:JDS_TTS_TASK_TYPE];
        instance.taskTypeVal = val;
    } else {
        instance.taskTypeVal = kTaskDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_ENGINE_TYPE]) {
        NSInteger val = [userDefaults floatForKey:JDS_TTS_ENGINE_TYPE];
        instance.ttsEngineType = val;
    } else {
        instance.ttsEngineType = kEngineDef;
    }

#ifdef __TEST
    // Testing
    if ([instance.appIDVal isEqualToString:@""]) {
        instance.appIDVal = kApplicationIdTestDef;
    }
    if ([instance.serverURLVal isEqualToString:@""]) {
        instance.serverURLVal = kServerURLTestDef;
    }
    if ([instance.appKeyVal isEqualToString:@""]) {
        instance.appKeyVal = kApplicationKeyTestDef;
    }
    if ([instance.secretKeyVal isEqualToString:@""]) {
        instance.secretKeyVal = kSecretKeyTestDef;
    }
#endif
    
    //NSLog(@"*-userConfigs exit2-*");
}

#pragma mark --Settings

// 更改配置并且存储到默认设置

- (IBAction)sampleRateDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.sampleRateChoice.selectedSegmentIndex forKey:JDS_TTS_SAMPLE_RATE];
    [userDefaults synchronize];
}

- (IBAction)APPIDEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.APPID.text forKey:JDS_TTS_APP_ID];
    [userDefaults synchronize];
}

- (IBAction)APPKeyEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.AppKey.text forKey:JDS_TTS_APP_KEY];
    [userDefaults synchronize];
}

- (IBAction)SecretKeyEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.SecretKey.text forKey:JDS_TTS_SECRET_KEY];
    [userDefaults synchronize];
}

- (IBAction)serverURLEdit:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:self.serverURL.text forKey:JDS_TTS_SERVER_URL];
    [userDefaults synchronize];
}

- (IBAction)languageDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.language.selectedSegmentIndex forKey:JDS_TTS_LANGUAGE];
    [userDefaults synchronize];
}

- (IBAction)timbreDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.timbre.selectedSegmentIndex forKey:JDS_TTS_TIM_TYPE];
    [userDefaults synchronize];
}

- (IBAction)aueDidChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:self.aueChoice.selectedSegmentIndex forKey:JDS_TTS_AUE_TYPE];
    [userDefaults synchronize];
}

- (IBAction)setVolume:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setFloat:self.volumeSlider.value forKey:JDS_TTS_VOLUME_LEVEL];
    self.volumeRes.text = [NSString stringWithFormat:@"%.1f", self.volumeSlider.value];
    [userDefaults synchronize];
}

- (IBAction)setSpeed:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setFloat:self.speedSlider.value forKey:JDS_TTS_SPEED_LEVEL];
    self.speedRes.text = [NSString stringWithFormat:@"%.1f", self.speedSlider.value];
    [userDefaults synchronize];
}

- (IBAction)timeStampSwitch:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.timeStampSwitch.on forKey:JDS_TTS_TIME_STAMP];
    [userDefaults synchronize];
}

- (IBAction)taskChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.taskChoice.selectedSegmentIndex forKey:JDS_TTS_TASK_TYPE];
    [userDefaults synchronize];
}

- (IBAction)engineChoice:(id)sender {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setBool:self.engineType.selectedSegmentIndex forKey:JDS_TTS_ENGINE_TYPE];
    [userDefaults synchronize];
}

- (IBAction)defaultSet:(id)sender {
    [self mySetDefault];
}

// 恢复到默认设置
- (void)mySetDefault {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];

    self.sampleRateChoice.selectedSegmentIndex = 3;
    [userDefaults setInteger:self.sampleRateChoice.selectedSegmentIndex forKey:JDS_TTS_SAMPLE_RATE];
    
    self.APPID.text = kApplicationIdDef;
    [userDefaults setObject:self.APPID.text forKey:JDS_TTS_APP_ID];
    
    self.AppKey.text = kApplicationKeyDef;
    [userDefaults setObject:self.AppKey.text forKey:JDS_TTS_APP_KEY];
    
    self.SecretKey.text = kSecretKeyDef;
    [userDefaults setObject:self.SecretKey.text forKey:JDS_TTS_SECRET_KEY];
    
    self.serverURL.text = kServerURLDef;
    [userDefaults setObject:self.serverURL.text forKey:JDS_TTS_SERVER_URL];

    self.language.selectedSegmentIndex = 0;
    [userDefaults setInteger:self.language.selectedSegmentIndex forKey:JDS_TTS_LANGUAGE];
    
    self.timbre.selectedSegmentIndex = kTimDef;
    [userDefaults setInteger:self.timbre.selectedSegmentIndex forKey:JDS_TTS_TIM_TYPE];
    
    self.aueChoice.selectedSegmentIndex = kAueDef;
    [userDefaults setInteger:self.aueChoice.selectedSegmentIndex forKey:JDS_TTS_AUE_TYPE];
    
    self.volumeSlider.value = kVolumeLevelDef;
    [userDefaults setInteger:self.volumeSlider.value forKey:JDS_TTS_VOLUME_LEVEL];
    self.volumeRes.text = [NSString stringWithFormat:@"%.1f", self.volumeSlider.value];
    
    self.speedSlider.value = kSpeedLevelDef;
    [userDefaults setInteger:self.speedSlider.value forKey:JDS_TTS_SPEED_LEVEL];
    self.speedRes.text = [NSString stringWithFormat:@"%.1f", self.speedSlider.value];
    
    self.timeStampSwitch.on = (BOOL)kTimeStampDef;
    [userDefaults setBool:self.timeStampSwitch.on forKey:JDS_TTS_TIME_STAMP];
    
    self.taskChoice.selectedSegmentIndex = kTaskDef;
    [userDefaults setInteger:self.taskChoice.selectedSegmentIndex forKey:JDS_TTS_TASK_TYPE];
    
    self.engineType.selectedSegmentIndex = kEngineDef;
    [userDefaults setInteger:self.engineType.selectedSegmentIndex forKey:JDS_TTS_ENGINE_TYPE];
    
    [userDefaults synchronize];
}

// 加载默认设置选项
- (void)myViewLoad {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    
    if([userDefaults objectForKey:JDS_TTS_SERVER_URL]) {
        self.serverURL.text = [userDefaults stringForKey:JDS_TTS_SERVER_URL];
    } else {
        self.serverURL.text = kServerURLDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_APP_ID]) {
        self.APPID.text = [userDefaults stringForKey:JDS_TTS_APP_ID];
    } else {
        self.APPID.text = kApplicationIdDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_APP_KEY]) {
        self.AppKey.text = [userDefaults stringForKey:JDS_TTS_APP_KEY];
    } else {
        self.AppKey.text = kApplicationKeyDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SECRET_KEY]) {
        self.SecretKey.text = [userDefaults stringForKey:JDS_TTS_SECRET_KEY];
    } else {
        self.SecretKey.text = kSecretKeyDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_SAMPLE_RATE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_SAMPLE_RATE];
        [self.sampleRateChoice setSelectedSegmentIndex:index];
    } else {
        [self.sampleRateChoice setSelectedSegmentIndex:3];
    }
    
    if([userDefaults objectForKey:JDS_TTS_LANGUAGE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_LANGUAGE];
        [self.language setSelectedSegmentIndex:index];
    } else {
        [self.language setSelectedSegmentIndex:0];
    }
    
    if([userDefaults objectForKey:JDS_TTS_TIM_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_TIM_TYPE];
        [self.timbre setSelectedSegmentIndex:index];
    } else {
        [self.timbre setSelectedSegmentIndex:kTimDef];
    }
    
    if([userDefaults objectForKey:JDS_TTS_AUE_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_AUE_TYPE];
        [self.aueChoice setSelectedSegmentIndex:index];
    } else {
        [self.aueChoice setSelectedSegmentIndex:kAueDef];
    }
    
    if([userDefaults objectForKey:JDS_TTS_VOLUME_LEVEL]) {
        float val = [userDefaults floatForKey:JDS_TTS_VOLUME_LEVEL];
        self.volumeSlider.value = val;
    } else {
        self.volumeSlider.value = kVolumeLevelDef;
    }
    self.volumeRes.text = [NSString stringWithFormat:@"%.1f", self.volumeSlider.value];
    
    if([userDefaults objectForKey:JDS_TTS_SPEED_LEVEL]) {
        float val = [userDefaults floatForKey:JDS_TTS_SPEED_LEVEL];
        self.speedSlider.value = val;
    } else {
        self.speedSlider.value = kSpeedLevelDef;
    }
    self.speedRes.text = [NSString stringWithFormat:@"%.1f", self.speedSlider.value];
    
    if([userDefaults objectForKey:JDS_TTS_TIME_STAMP]) {
        BOOL val = [userDefaults floatForKey:JDS_TTS_TIME_STAMP];
        self.timeStampSwitch.on = val;
    } else {
        self.timeStampSwitch.on = (BOOL)kTimeStampDef;
    }
    
    if([userDefaults objectForKey:JDS_TTS_TASK_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_TASK_TYPE];
        [self.taskChoice setSelectedSegmentIndex:index];
    } else {
        [self.taskChoice setSelectedSegmentIndex:kTaskDef];
    }

    if([userDefaults objectForKey:JDS_TTS_ENGINE_TYPE]) {
        NSInteger index = [userDefaults integerForKey:JDS_TTS_ENGINE_TYPE];
        [self.engineType setSelectedSegmentIndex:index];
    } else {
        [self.engineType setSelectedSegmentIndex:kEngineDef];
    }
}

#pragma mark - write to file

#ifdef __SAVE_TO_FILE
- (void) creatOutFile {
    // 获取Documents路径
    NSArray *path = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [path objectAtIndex:0];
    
    // 获取文件路径
    NSTimeInterval timestamp = [NSDate timeIntervalSinceReferenceDate];
    _outfile = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"ttsAudio-%f.pcm",timestamp]];
    NSLog(@"Output file:%@", _outfile);
    
    // 如果文件不存在
    if (![[NSFileManager defaultManager] fileExistsAtPath:_outfile])
    {
        [[NSFileManager defaultManager] createFileAtPath:_outfile contents:nil attributes:nil];
    }
}

- (void) outToFile:(NSData *)data {
    if (data) {
        NSFileHandle *fileHandle = [NSFileHandle fileHandleForUpdatingAtPath:_outfile];
        [fileHandle seekToEndOfFile];
        [fileHandle writeData:data];
        [fileHandle closeFile];
    }
}
#endif

#pragma mark - util functions

// Dictionary to JSON
+ (NSString *)convertToJsonData:(NSDictionary *)dict {
    
    NSError *error;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:dict options:NSJSONWritingPrettyPrinted error:&error];
    
    NSString *jsonString;
    if (!jsonData) {
        NSLog(@"%@",error);
        return nil;
    }else{
        jsonString = [[NSString alloc]initWithData:jsonData encoding:NSUTF8StringEncoding];
    }
    
    return jsonString;
}

// JSON to dictionary
+ (NSDictionary *)dictionaryWithJsonString:(NSString *)jsonString {
    if (jsonString == nil) {
        return nil;
    }
    
    NSData *jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    NSError *err;
    NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:jsonData
                                                        options:NSJSONReadingMutableContainers
                                                          error:&err];
    if(err)
    {
        NSLog(@"json解析失败：%@",err);
        return nil;
    }
    return dic;
}

@end
